#include <math.h>
#include "Accord.h"
#include <stdlib.h>
#include <string.h>
#include <libintl.h>
#define _(String) gettext (String)

const double CAccord::m_fNoteEtalon[18]=
{
	//Valeur des fréquences des notes de la portée à 0
	32.703203125,	//DO 0 etc...
	34.6472125,
	34.6472125,
	36.708125,
	38.890859375,
	38.890859375,
	41.2034375,
	43.653515625,
	46.249296875,
	46.249296875,
	48.9994523125,
	51.913125,
	51.913125,
	55,
	58.27046875,
	58.27046875,
	61.735390625,
	61.735390625
};


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAccord::CAccord()
{
	//Constructeur par défaut
	Initialisation();	//Initialisation des variables

}

CAccord::~CAccord()
{
	MapIndexNote.clear();   //Mise à zéro du vecteur
}

CAccord::CAccord(std::string NomAccord)
{
	//Constructeur avec nom de l'accord et des notes
	Initialisation();	//Initialisation des variables
	m_szNomAccord=NomAccord;	//Mise à jour du nom de l'accord
}

double CAccord::GetValeurNote(int Index)
{
	return m_fNotes[Index];	//Retourne la fréquence de la note suivant sa position
}

void CAccord::SetAccord(std::string NomAccord)
{
	//Mise à jour de l'accord
	m_szNomAccord=NomAccord;	//Initialisation des variables
}

std::string CAccord::GetNomAccord()
{
	return m_szNomAccord;	//Retourne le nom de l'accord
}

std::string CAccord::GetNomNote(int Index)
{
    if ((Index>=0) and (Index<6))
        return m_szarNotes[Index];	//Retourne le nom de la note en fonction de sa position
    else
        return "";
}

bool CAccord::CalculNotes(void)
{
	//Mise à jour des fréquences des 6 notes
	bool retour=true;
	std::string Note;
	int Harmo,comparaison=1;
	unsigned int j;
	char buffer[10],szHarmo;

	for (int i=0;((i<6) && (retour)) ;i++)	//Pour les 6 notes
	{
	    Note=m_szarNotes.at(i);
	    szHarmo=Note[Note.size()-1];    //Nom de la note
	    Harmo=atoi((const char*)&Note[Note.size()-1]); //Hauteur de la note

	    memset(buffer,'\0',10); //Mise à zéro du buffer
	    j=0;

	    Note.copy(buffer,Note.size()-1);    // copie le nom de la note dans le buffer
		while((comparaison!=0) && (j<MapIndexNote.size()-1))
			//Tant que l'on n'a pas trouvé la Note et pas à la fin du tableau
		{
			comparaison=MapIndexNote[j].compare(buffer);	//Récupération de la note, de l'index et de la prochaine position
			j++;
		}

		if (comparaison!=0)	//Si la note trouvée n'est pas celle recherchée
		{
			retour=false;
		}
		else
		{
			//Autrement sauvegarde du nom de la note et de sa fréquence

			m_fNotes[i]=(double)m_fNoteEtalon[j-1]*pow(2,Harmo);
		}
		comparaison=1;
	}
	return retour;
}

void CAccord::Initialisation()
{
	//Initialise tous les noms des notes
     MapIndexNote.push_back("DO");
     MapIndexNote.push_back("DO#");
     MapIndexNote.push_back("REb");
     MapIndexNote.push_back("RE");
     MapIndexNote.push_back("RE#");
     MapIndexNote.push_back("MIb");
     MapIndexNote.push_back("MI");
     MapIndexNote.push_back("FA");
     MapIndexNote.push_back("FA#");
     MapIndexNote.push_back("SOLb");
     MapIndexNote.push_back("SOL");
     MapIndexNote.push_back("SOL#");
     MapIndexNote.push_back("LAb");
     MapIndexNote.push_back("LA");
     MapIndexNote.push_back("LA#");
     MapIndexNote.push_back("SIb");
     MapIndexNote.push_back("SI");
     MapIndexNote.push_back("DOb");

}

int CAccord::GetNoteInf(double Freq)
{
	//Retourne l'index de la note de l'accord immédiatement inférieur (-1 si la note est
	// inférieure à la plus basse note de l'accord et 5 si la fréquence est supérieure à
	// la note la plus haute de l'accord

	int IndexNote;	//l'index de la note de l'accord
	int i=0;

	//tant que la valeur de la note de l'accord est inférieure et que l'on n'a pas parcouru l'ensemble des notes
	while ((i<6) && (GetValeurNote(i)<Freq))
		i++;

	IndexNote=i-1;	//l'index mis à jour

	return IndexNote;	//Retourne l'index

}

int CAccord::GetNoteSup(double Freq)
{
	//Retourne l'index de la note de l'accord immédiatement supérieur
	// 6 si la fréquence est supérieure à la plus haute note de l'accord

	int IndexNote;

	int i=0;

	//tant que la valeur de la note de l'accord est inférieure et que l'on n'a pas parcouru l'ensemble des notes
	while ((i<6) && (GetValeurNote(i)<Freq))
		i++;

	IndexNote=i;

	return IndexNote;	//Retourne l'index
}

void CAccord::SetNomNote(unsigned int Index,std::string Note)
{
    // Met à jour le nom de la note
    if ((Index>=0) and (Index<6))   //Si l'index de la note est sans les tolérances
    {
        if ((Index<=m_szarNotes.size()-1) && (!m_szarNotes.empty()))    // Si la note n'est pas la dernière ou que le vecteur des notes n'est pas vide
        {
            std::vector<std::string>::iterator Iter;    //Itération sur le vecteur

            Iter=m_szarNotes.begin();   // du début
            Iter+=Index;    // récupération de l'itération pour l'index donné

            *Iter=Note; //mise à jour du nom
        }
        else if (Index==m_szarNotes.size()) //Si nouvelle note
        {
            m_szarNotes.push_back(Note);    //sauvegarde dans le vecteur
        }
    }
}
