#include "Proprietes.h"
#include "Gtk_Tuner.h"
#include "FFT.h"
#include <libintl.h>
#define _(String) gettext (String)
#ifdef HAVE_CONFIG_H
  #include "config.h"
#endif //HAVE_CONFIG_H
#ifdef _WIN32
#include <windows.h>
#include <mmsystem.h>
#include <direct.h>
#endif //_WIN32

#include <glib.h>

#ifdef __linux__
snd_pcm_t *handle;  //Handle de la carte son
short int  *buf; //buffer des échantillons
#elif defined(_WIN32)
HWAVEIN* Handle;	//Structure du Handle
WAVEHDR Buffer;	//Structure du buffer
#endif

//Analyse du signal
int Gtk_Tuner::AnalyseSignal(void)
{
    CFFT fft;   //Variable pour l'analyse du signal
    short int echantillon;

#ifdef __linux__
    unsigned long Echantillonslus;
    snd_pcm_sframes_t lues,en_attente;  //Nb d'échantillons lues et en attente d'etre lues
    en_attente=snd_pcm_avail_update(handle);    //Récupération du nombre d'échantillons en attente
    if ((unsigned int)en_attente > taillebuffer) //Si Nb d'échantillons supérieur au buffer
       en_attente=taillebuffer;   //Alors Echantillons égal au taille du buffer
#elif defined (_WIN32)
   
    waveInAddBuffer(*Handle,&Buffer,sizeof(Buffer));

#endif

#ifdef __linux__   

    if (en_attente>=0)  //Si il y a des échantillons
    {
	 
	buf=(short int*)malloc(en_attente*periode*2);   //Réservation de mémoire pour le buffer*/
	
        lues=snd_pcm_readi(handle,buf,en_attente); //Lecture des échantillons

        if (lues>0)    //Si on a réussi à les lire
        {

	     Echantillonslus=lues/periode;
	     
	     //Déplacement des anciens échantillons
	    for(long i=Echantillonslus; i<param_xml.nNbreEchantillon;i++)
	     {
		  RealIn[i-Echantillonslus]=RealIn[i];
		  ImagIn[i-Echantillonslus]=0;

	     }	    
	    //Sauvegarde des échantillons du buffer
	    for (unsigned long i=0; (i/periode<Echantillonslus) && ((i/periode)<param_xml.nNbreEchantillon);i+=periode)
	    {
		 echantillon=*(buf+i);		  
		 RealIn[i/periode+param_xml.nNbreEchantillon-Echantillonslus]=(double)echantillon;
		 ImagIn[i/periode+param_xml.nNbreEchantillon-Echantillonslus]=0;
	    }
	    
	    
	}
	else    //si on n'arrive pas à les lire
	{
	     snd_pcm_prepare(handle); //Réinitialisation du pipe
	}
	free(buf);
	buf=NULL;
    }

    else    //si on n'arrive pas à les lire
    {
	 snd_pcm_prepare(handle); //Réinitialisation du pipe
    }
#elif defined _WIN32
    //Pour tous les échantillons lues et si on dépasse pas le nb d'échantillons total


    for (unsigned long int i=0; i<(param_xml.nNbreEchantillon*periode);i+=periode)
    {
	 echantillon=Buffer.lpData[i]+Buffer.lpData[i+1]*256;

	 RealIn[i/periode]=(double)echantillon;  //Sauvegarde de l'échantillon dans le tableau                
    }

#endif

   
    bAffichageSpectre=true;

    //Application de la transformée de Fourier aux échantillons lues
    fft.fft_float(param_xml.nNbreEchantillon,0,RealIn,ImagIn,RealO,ImagO);

    //Maj des fréquences
    MAJFrequence();

    //Mise à jour des Slider
    if (param_xml.nInterface_Affichage==0)
	 MAJSlider();
    else if (param_xml.nInterface_Affichage==1)
    {
	 //mise à jour de l'affichage rond
	 Glib::RefPtr<Gdk::Window> win = m_Affichage_Rond.get_window();
	 if (win)
	 {
	      Gdk::Rectangle r(0, 0, m_Affichage_Rond.get_allocation().get_width(),
			       m_Affichage_Rond.get_allocation().get_height());
	      win->invalidate_rect(r, false);
	 }

    }

    if (param_xml.bVisuFreq)
    {
	 //Affichage du spectre fréquentiel si c'est désiré

	 Glib::RefPtr<Gdk::Window> win = m_AffichageFFT->get_window();
	 if (win)
	 {
	      Gdk::Rectangle r(0, 0, m_AffichageFFT->get_allocation().get_width(),
			       m_AffichageFFT->get_allocation().get_height());
	      win->invalidate_rect(r, false);
	 }
       
    }	       
       

    return true;

}

//Constructeur
Gtk_Tuner::Gtk_Tuner(const Glib::RefPtr<Gnome::Glade::Xml> & refGlade): m_refGlade(refGlade),adjustement(0,0,1000,1,10,10),Adjustment_precis(0,0,100,1,10,10)
 {

    
    std::string chemin_icone; //Chemin de l'icone

    ImagO=NULL;
    RealO=NULL;
    ImagIn=NULL;
    RealIn=NULL;

#ifdef __linux__
    char *home; //Pointeur char sur le chemin home
    home=getenv("HOME");    //on retrouve le chemin /home/utilisateur

    if (home!=NULL) //Si cette variable n'est pas nulle
    {

        m_szparametres_xml=home;
        m_szHome=home;  //Mise à jour de la variable du chemin du home

        m_szparametres_xml+="/.guitar_tuner/guitar_tuner.xml";  //Chemin du fichier
        m_szHome+="/.guitar_tuner"; //Chemin du dossier

    }
    else
    {
        //Sinon affichage du message d'erreur
	 std::string message(_("The variable named $HOME doesn't exist\n"));

	 message+=_("Create this variable");

        Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
        Dialog.set_title(_("Environment variables problem"));
        Dialog.run();

        on_wndGuitarTuner_hide();   //quitte l'application

    }

#elif defined (_WIN32)
    unsigned long resultat;
    char cHome[MAX_PATH];
//Pour windaube on retrouve le chemin courant de l'application
    resultat=GetCurrentDirectory(MAX_PATH,cHome);

    m_szHome=cHome;

    if (resultat)
    {
	 //Ainsi que le chemin du fichier guitar.xml
	 m_szparametres_xml=m_szHome;
	 m_szparametres_xml+="\\guitar_tuner.xml";
    }
    else
    {
	 //Sinon affichage du message d'erreur
	 std::string message(_("Problem to find the software path\n")); 
	 message+=_("The software stopped");

	 Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
	 Dialog.set_title(_("Directory problem"));
	 Dialog.run();

	 on_wndGuitarTuner_hide();   //quitte l'application
    }

#endif


//Mise à jour de la variable pour le niveau des raies
    param_affichage.Niveau_Max=0;
//Création des différents menus
     m_refActionGroup=Gtk::ActionGroup::create();
     m_refActionGroup->add(Gtk::Action::create("FileMenu",_("File")));
     m_refActionGroup->add(Gtk::Action::create("FileQuit",Gtk::Stock::QUIT),sigc::mem_fun(*this,&Gtk_Tuner::on_Quit));
     m_refActionGroup->add(Gtk::Action::create("Edition",_("Edit")));
     m_refActionGroup->add(Gtk::Action::create("Preferences",Gtk::Stock::PREFERENCES,"",_("Settings configuration")),sigc::mem_fun(*this,&Gtk_Tuner::on_menuProprietes));
     m_refActionGroup->add(Gtk::Action::create("Aide",_("Help")));
     m_refActionGroup->add(Gtk::Action::create("A_propos",Gtk::Stock::ABOUT),sigc::mem_fun(*this,&Gtk_Tuner::on_menuAPropos));
     m_refActionGroup->add(Gtk::Action::create("Analyse",Gtk::Stock::MEDIA_PLAY,"","Analyse"),sigc::mem_fun(*this,&Gtk_Tuner::onbuttonanalyser));
     m_refActionGroup->add(Gtk::Action::create("Arret",Gtk::Stock::MEDIA_STOP,"","Stop"),sigc::mem_fun(*this,&Gtk_Tuner::onbuttonarret));

     m_refUIManager=Gtk::UIManager::create();
     m_refUIManager->insert_action_group(m_refActionGroup);
     add_accel_group(m_refUIManager->get_accel_group());
 Glib::ustring ui_info =
    "<ui>"
    "   <menubar name='MenuBar'>"
    "       <menu action='FileMenu'>"
    "           <menuitem action='FileQuit'/>"
    "       </menu>"
    "       <menu action='Edition'>"
    "           <menuitem action='Preferences'/>"
    "       </menu>"
    "       <menu action='Aide'>"
    "           <menuitem action='A_propos'/>"
    "       </menu>"
    "   </menubar>"
    "   <toolbar name='ToolBar'>"
    "     <toolitem action='Analyse'/>"
    "     <toolitem action='Arret'/>"
    "     <toolitem action='Preferences'/>"
    "  </toolbar>"
    "</ui>";

#ifdef GLIBMM_EXCEPTIONS_ENABLED
 std::string message(_("Problem to create menus"));
try
{
     m_refUIManager->add_ui_from_string(ui_info);
}
catch(const Glib::Error &ex)
{
     Gtk::MessageDialog::MessageDialog Dialog(ex.what(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
     Dialog.set_title(message);
     Dialog.run();   // affichage d'une boite de dialogue relatant le problème
}
#else
std::auto_ptr<Glib::Error> ex;
m_refUIManager->add_ui_from_string(ui_info,ex);
if (ex.get())
{
     std::cerr<<message<<ex->what());
}
#endif //GLIBMM_EXCEPTIONS_ENABLED
//Mise à jour des pointeurs pour le menu et la barre d'outils
     Gtk::Widget *pMenuBar=m_refUIManager->get_widget("/MenuBar");
if (pMenuBar)
     Vbox1.pack_start(*pMenuBar,Gtk::PACK_SHRINK);

Gtk::Widget *pToolbar=m_refUIManager->get_widget("/ToolBar");
if (pToolbar)
{
     Vbox1.pack_start(*pToolbar,Gtk::PACK_SHRINK);

}

#ifdef __linux__
    buf=NULL;   //Pas de buffer
#endif
   
    bAffichageSpectre=false;    //Pas de fenetre spectrale
    m_wndSpectre=NULL;  //Pointeur sur la fenetre NULL

    set_resizable(false);
//Au cas ou le fichier config.h est trouvé
#ifdef HAVE_CONFIG_H
    {
//Les chemins sont dans la variable GLADE_DIR
	m_szchemin_share=GLADE_DIR;

	chemin_icone=( m_szchemin_share+"/Guitare.svg");
    }
#else
    {
//Sinon on prend le chemin courant
	 char chemin_courant[PATH_MAX];
	 getcwd(chemin_courant,PATH_MAX);
	 m_szchemin_share=chemin_courant;
#ifdef __linux__
	 m_szchemin_share+="/share";
	 chemin_icone=( m_szchemin_share+"/Guitare.svg");

#elif defined (_WIN32)
//Pour windaube on prend le chemin courant
    m_szchemin_share+="\\share";
    chemin_icone=(m_szchemin_share +"\\Guitare.png");
#endif
    }

#endif
//Création des différents contextes pour l'affichage des fréquences, carte son etc..
for(int i=0;i<5;i++)
{
     std::string contexte;
     std::ostringstream o_stream;

     contexte="ctx";
     o_stream<<i;
     contexte+=o_stream.str();
     stsInfo[i].set_has_resize_grip(false);
     Hbox6.pack_start(stsInfo[i]);
     ctx[i]= stsInfo[i].get_context_id(contexte);
}
//Connection du signal hide à la procédure on_wndGuitarTuner_hide
signal_hide().connect(sigc::mem_fun(*this,&Gtk_Tuner::on_wndGuitarTuner_hide)),
//intégration de la boite des contextes à la boite principale
Vbox1.pack_start(Hbox6,Gtk::PACK_SHRINK);
//Ajout de la boite principale à la fenêtre
    add(Vbox1);
//Mise à jour de l'icône pour la fenêtre
    set_default_icon_from_file(chemin_icone);
    set_title(_("Guitar Tuner"));
//La fenêtre principale toujours au dessus
    set_keep_above(true);
    show_all_children();


}
//Affichage de l'interface ronde
void Gtk_Tuner::Affichage_Rond(void)
{

     Vbox1.pack_start(m_Affichage_Rond);

     m_Affichage_Rond.signal_expose_event().connect(sigc::mem_fun(*this,&Gtk_Tuner::on_DrawingRond));

     Vbox1.reorder_child(Hbox6,-1);

     set_size_request(800,600);

     show_all_children();

}

//Affichage de l'interface standard
void Gtk_Tuner::Affichage_Standard(void)
{
     std::string szchemin_image;

     set_size_request(1030,460);

     if (Vbox3.get_children().empty())
     {

	  Vbox3.pack_start(Hbox1);
	  Vbox3.pack_start(Hbox2);
	  Vbox3.pack_start(Hbox3);
	  Vbox3.pack_start(Hbox4);
	  Vbox3.pack_start(Hbox5);
#ifdef __linux__

	  szchemin_image=(m_szchemin_share+"/repere.png");
#elif defined _WIN32

	  szchemin_image=(m_szchemin_share+"\\repere.png");
#endif

	  for(int i=0;i<6;i++)
	  {
	       Hbox1.pack_start(Eventbox[i]);
	       Eventbox[i].add(lblNotes[i]);
	       Image[i].set(szchemin_image);
	       Hbox2.pack_start(Image[i]);
	  }

	  Hbox3.pack_start(hsNiveau);
	  hsNiveau.set_draw_value(false);
	  hsNiveau.set_adjustment(adjustement);


	  Hbox4.pack_start(Eventbox[6]);
	  Eventbox[6].add(Image[6]);
	  Image[6].set(szchemin_image);

	  Hbox5.pack_start(hsPrecis);
	  hsPrecis.set_draw_value(false);
	  hsPrecis.set_adjustment(Adjustment_precis);

     }

        //On retrouve le style de la boite d'évènement pour changer la couleur par la suite
     style=Eventbox[0].get_modifier_style();


        //On retrouve les dimensions de l'image des index

     Vbox1.pack_start(Vbox3);

     Vbox1.reorder_child(Hbox6,-1);

     show_all_children();

}
//Arret de l'analyse
void Gtk_Tuner::onbuttonarret()
{

    if (pthread.connected()) //Si le thread existe
    {

        pthread.disconnect();
	m_refActionGroup->get_action("Analyse")->set_sensitive(true);
	m_refActionGroup->get_action("Preferences")->set_sensitive(true);

        bAffichageSpectre=false;    //Pas de fenetre spectrale
    }
#ifdef __linux__
    if (buf)    //Si le buffer existe
    {
        free(buf);  //Libération de la mémoire
        buf=NULL;   //Mise à zéro de sa valeur
    }

#elif defined (_WIN32)
    if (Handle!=NULL) //Si on avait déjà un Handle pour la carte son
    {

        waveInStop(*Handle); //On arrête l'enregistrement
    }
#endif
    if (m_wndSpectre)
    {
            m_wndSpectre->hide();   //fermeture de la fenetre spectrale
    }
   
    //Mise à jour de la variable niveau
    param_affichage.Niveau_Max=0;

//Suppression des différents tableaux pour la transformée de Fourier
    if (ImagO!=NULL)
    {
	 delete ImagO;
	 ImagO=NULL;
    }


    if (RealO!=NULL)
    {
	 delete RealO;
	 RealO=NULL;
    }

    if (ImagIn!=NULL)
    {
	 delete ImagIn;
	 ImagIn=NULL;
    }

    if (RealIn!=NULL)
    {
	 delete RealIn;
	 RealIn=NULL;
    }

}

//Lors de l'appui du bouton propriétés

void Gtk_Tuner::onproprietes()
{
        if (!bCarteSon) //S'il n'y a pas de carte son
        {
            on_wndGuitarTuner_hide();   //Arret de l'application

        }
        else
        {

            if (!bAccord)   //Meme chose s'il n'y a pas de fichier accord
            {
		 std::string message(_("Problem to find the chords file\n"));
		 message+=_("The software stopped");

                Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                Dialog.set_title(_("Problem to open the chords file"));
                Dialog.run();

                on_wndGuitarTuner_hide();

            }
            else
            {
                int reponse;    //Variable sur la réponse de la boite de dialogue Paramètres

                if (param_xml.bFichierOK==false)    //S'il n'y a pas de fichier paramètres
                {
                    param_xml.nNbreEchantillon=16384;   //Mise à jour des variables par défaut
                    param_xml.bVisuFreq=false;
                    param_xml.nValeurAttenuation=5;
                    param_xml.nNumero_carte_son=0;
                    param_xml.nFreqEchantillonnage=24000;
                    param_xml.nNumeroAccord=0;
		    param_xml.nInterface_Affichage=0;
		    param_xml.bVisuRepere=false;
                }
#ifdef __linux__
                if (handle!=NULL)   //S'il y a un handle de la carte son
                {
                    snd_pcm_close(handle);  //Fermeture de la carte son
                    handle=NULL;    //handle à nul
                }
#elif defined (_WIN32)
                if (Handle!=NULL)
                {
                    waveInStop(*Handle); //On arrête l'enregistrement
                    waveInUnprepareHeader(*Handle,&Buffer,sizeof(Buffer));	//On nettoie le buffer
                    Handle=NULL;
                }
#endif
		m_dlgProprietes->SetCheminShare(m_szchemin_share);
                m_dlgProprietes->SetParametres(param_xml);  //Mise à jour des paramètres
                m_dlgProprietes->SetCheminParam_XML(m_szparametres_xml);    //et du chemin du fichier


                reponse=m_dlgProprietes->run();  //Affichage de la boite


                if ((reponse!=1) && (!param_xml.bFichierOK))   //SI appui sur annuler et pas de fichier paramètres
                {
                    m_dlgProprietes->hide();
                    std::string message(_("The settings are unknown\n"));
		    message+=_("The analyse is impossible");
                    Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                    Dialog.set_title(_("Unknown settings"));
                    Dialog.run();

                    m_refActionGroup->get_action("Analyse")->set_sensitive(false);   //Pas d'analyse possible
                }
                else
                    MAJControles();

        }

    }

}

//Lors du clic sur Analyser
void Gtk_Tuner::onbuttonanalyser()
{
//On retrouve l'emplacement de la première image des index
     alloc=Image[0].get_allocation();

     nPositionPremierIndex=Image[0].get_allocation().get_width()/2;
     nDistanceInterIndex=Image[1].get_allocation().get_x()-Image[0].get_allocation().get_x();

//Création des tableaux pour la transformée de Fourier
     if (ImagO==NULL)
	  ImagO=new double[param_xml.nNbreEchantillon];
     if (RealO==NULL)
	  RealO=new double[param_xml.nNbreEchantillon];
     if (ImagIn==NULL)
	  ImagIn=new double[param_xml.nNbreEchantillon];
     if (RealIn==NULL)
	  RealIn=new double[param_xml.nNbreEchantillon];

     if ((ImagO!=NULL) && (RealO!=NULL) && (ImagIn!=NULL) && (RealIn!=NULL))
     {

	  for (unsigned long int i=0; i<param_xml.nNbreEchantillon;i++)
	  {
	       ImagO[i]=0;
	       RealO[i]=0;
	       ImagIn[i]=0;
	       RealIn[i]=0;
	  }
	  param_affichage.Frequence_Pic=0;

#ifdef __linux__
	  int err;
	 
	  //Préparation de la carte son
	  if ( snd_pcm_prepare(handle)!=0)
	  {
	       std::string message(_("Problem to prepare sound card"));

	       Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
	       Dialog.set_title(_("Preparation problem"));
	       Dialog.run();
	  }
	  else
	  {

	       //Attente des premiers échantillons
	       if ((err=snd_pcm_wait(handle,1000))<0)
	       {
		    std::string message(_("The sound card is not ready\n"));

		    message+=snd_strerror(err);

		    Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
		    Dialog.set_title(_("Sound card preparation"));
		    Dialog.run();
	       }
	       else
	       {

		    if (snd_pcm_start(handle)>=0)    //Démarrage de la lecture
		    {
			 Demarrage_thread();

		    }
		    else
		    {
			 std::string message(_("Problem to record the samples"));

			 message+=snd_strerror(err);

			 Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
			 Dialog.set_title(_("Record problem"));
			 Dialog.run();
		    }


	       }

	  }
#elif defined _WIN32
	  MMRESULT resultat; //Resultat de l'opération

	  resultat=waveInStart(*Handle); //début de l'enregistrement
	  if (resultat==MMSYSERR_NOERROR)
	       Demarrage_thread();
	  else
	  {
	       std::string message(_("Problem to record the samples"));


	       Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
	       Dialog.set_title(_("Record problem"));
	       Dialog.run();
	  }

#endif
     }
     else
     {
	  std::string message(_("Memory allocation problem"));


	  Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
	  Dialog.set_title(_("Memory problem"));
	  Dialog.run();
     }

}
//Démarrage du thread
void Gtk_Tuner::Demarrage_thread(void)
{
     if (!pthread.connected())   //Si le thread n'existe pas
     {

	  //Création du thread
	  pthread=Glib::signal_idle().connect( sigc::mem_fun(*this, &Gtk_Tuner::AnalyseSignal),Glib::PRIORITY_HIGH_IDLE + 20  /*Glib::PRIORITY_DEFAULT_IDLE*/);

	  //Si le thread existe
	  if (pthread.connected())
	  {
	       m_refActionGroup->get_action("Analyse")->set_sensitive(false);
	       m_refActionGroup->get_action("Preferences")->set_sensitive(false);

	       //Si visualisation des fréquences est cochée
	       if (param_xml.bVisuFreq)
	       {

		    m_refGlade->get_widget("window2",m_wndSpectre);    //On rerouve le pointeur du bouton annuler
		    if (m_wndSpectre)
		    {
			 //Maximisation de la fenetre
			 m_wndSpectre->maximize();

			 //on retrouve les bornes des fréquences
			 nMini=Notes_Accords.GetValeurNote(0)-30;
			 nMaxi=Notes_Accords.GetValeurNote(5)+50;


			 m_refGlade->get_widget("drawingarea1",m_AffichageFFT);

			 if (m_AffichageFFT)
			 {
			      //Pour le rafraichissement de la fenetre
			      m_AffichageFFT->signal_expose_event().connect(sigc::mem_fun(*this,&Gtk_Tuner::on_DrawingSpectre));

			 }
			 //Démarrage de la fenetrre
			 Gtk::Main::run(*m_wndSpectre);

			 m_wndSpectre->show_all_children();

		    }
	       }
	  }
     }
}

//Destructeur
Gtk_Tuner::~Gtk_Tuner()
{
    onbuttonarret();

#ifdef __linux__

    if (handle!=NULL)
        snd_pcm_close(handle);
#elif defined(_WIN32)
    if (Handle!=NULL) //Si on avait déjà un Handle pour la carte son
    {

	 waveInStop(*Handle); //On arrête l'enregistrement

	 waveInUnprepareHeader(*Handle,&Buffer,sizeof(Buffer)); //On nettoie le buffer

	 delete Handle; //Suppression du Handle
	 delete Buffer.lpData; //et des données
    }

#endif

}

//Lors du changement d'accord
int Gtk_Tuner::Accord_changed()
{
    int ret;    //Retour d'erreur

    char *nomAccord;    //Pointeur sur le nom de l'accord
    xmlDocPtr xmlAccord_doc = NULL; //Pointeur sur le document
    xmlXPathContextPtr xmlparametres_context = NULL;    //Pointeur sur le contexte
    xmlXPathObjectPtr xmlobject_accord,xmlobject_nomAccord, xmlobject_nomNote;  //Pointeurs sur les objets
    xmlNodePtr node;    //Pointeur sur le noeud

    ret=1;
    //Récupération du pointeur sur le document
    xmlAccord_doc = xmlParseFile (m_szchemin_accord_xml.c_str());

    if (xmlAccord_doc)
    {
        //Initialisation des chemins
        xmlXPathInit ();

        //Récupération du contexte
        xmlparametres_context = xmlXPathNewContext (xmlAccord_doc);

        if (xmlparametres_context)
        {
            //Variable sur les chemin des accords
            std::string path_temp,path_accord("/accords/accord"),path_nom_accord;
            std::ostringstream Numero;

            Numero<<param_xml.nNumeroAccord+1;

            path_accord+=Numero.str();
            path_temp=path_accord;
            path_nom_accord=path_accord + "/nom/text()";    //Récupération du chemin du nom de l'aacord

            path_accord+="/text()"; // et du noeud des données de l'accord

            xmlobject_accord = xmlXPathEval ((const xmlChar*)path_accord.c_str(), xmlparametres_context);
            xmlobject_nomAccord = xmlXPathEval ((const xmlChar*)path_nom_accord.c_str(), xmlparametres_context);

            if (xmlobject_nomAccord->nodesetval->nodeNr)
            {
                //Récupération du nom de l'accord
                node=xmlobject_nomAccord->nodesetval->nodeTab[0];
                nomAccord=(char*)node->content;

                //Mise à jour de l'objet CAccord
                Notes_Accords.SetAccord(nomAccord);

                //Pour toutes les notes de l'accord
                for (int Index_Note=1;Index_Note<xmlobject_accord->nodesetval->nodeNr-1;Index_Note++)
                {
                    std::ostringstream Numero_note;   //Numéro de la note
                    std::string path_notes(path_temp);  //les différents chemin des notes

                    path_notes+="/note";

                    Numero_note<<Index_Note;

                    path_notes+=Numero_note.str();

                    path_notes+="/text()";  //CChemin de l'Objet

                    xmlobject_nomNote = xmlXPathEval ((const xmlChar*)path_notes.c_str(), xmlparametres_context);

                    if (xmlobject_nomNote->nodesetval->nodeNr)
                    {
                        //Récupération du nom de la note
                        node=xmlobject_nomNote->nodesetval->nodeTab[0];
                        Notes_Accords.SetNomNote(Index_Note-1, (const char*)node->content);
                    }

                }
                //Libération de la mémoire
                xmlXPathFreeObject(xmlobject_accord);
                xmlXPathFreeObject(xmlobject_nomAccord);
                xmlXPathFreeObject(xmlobject_nomNote);

                xmlXPathFreeContext(xmlparametres_context);

                //Mise à jour des fréquences des notes
                Notes_Accords.CalculNotes();


            }
            else
                ret=-1;

            //Mise à jour du texte des labels
            for (int IndexLabel=0;IndexLabel<6;IndexLabel++)
            {
                //Mise à jour des noms des labels des notes


                lblNotes[IndexLabel].set_text(Notes_Accords.GetNomNote(IndexLabel));


            }
	    if (param_xml.nInterface_Affichage==1)
		 show_all_children();

            xmlFreeDoc(xmlAccord_doc);  //Libération du pointeur sur le document
        }
        else
        {
	     std::string message(_("Problem to create a xml context"));
            Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
            Dialog.set_title("XML Problem");
            Dialog.run();

            ret=-1;
        }
    }
    else
    {
	 std::string message(_("Problem to load the chords file  "));
        message+=m_szchemin_accord_xml;
        Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
        Dialog.set_title(_("Problem chords file"));
        Dialog.run();

        ret=-1;
    }
    return ret;

}

//Lors de l'arret du programme
void Gtk_Tuner::on_wndGuitarTuner_hide()
{

    onbuttonarret();

    exit(0);

}

//Lors de la mise à jour de l'affichage rond
bool Gtk_Tuner::on_DrawingRond(GdkEventExpose *event)
{
     int retrait,position_y_niveau;
     Gdk::Color Noir;
     const int taille_trait=20;
     double xc,yc,xline,yline;
     int wp,hp;
     double rouge=0,jaune=0;

     Noir.set_rgb(0,0,0);
//On retrouve le pointeur de la fenêtre
     Glib::RefPtr<Gdk::Window> win = m_Affichage_Rond.get_window();
     if (win)
     {
//Création d'un cairo contexte
        Cairo::RefPtr<Cairo::Context> cr =win->create_cairo_context();
//On retrouve les dimensions de la fenêtre
        Gtk::Allocation allocation = m_Affichage_Rond.get_allocation();

        const int width = allocation.get_width();
        const int height = allocation.get_height();

        if (event)
        {
            //On retrouve la zone a dessiner
            cr->rectangle(event->area.x, event->area.y,
            width, height);
            cr->clip();
        }
        //On met toute la zone en blanc
        cr->save();
        cr->set_source_rgba(1, 1, 1, 0.9);
        cr->paint();
        cr->restore();
	cr->stroke();

	int pluspetit=MIN(width,height);

	cr->save();
	cr->arc(width/2,height/2,pluspetit/3,0,2.0*M_PI);
	cr->set_source_rgba(1,1,1,0.9);
	cr->fill_preserve();
	cr->restore();
	cr->stroke();

	cr->set_source_rgba(0, 0, 0, 0.9);
//Affichage des noms des notes, des traits sous le noms des notes, de la couleur des notes
	for (int i=0; i<6;i++)
	{
	     std::string sznote;
	     Glib::RefPtr<Pango::Layout> pl=Pango::Layout::create(cr);

	     Glib::RefPtr<Gdk::GC> gc;

	     sznote=Notes_Accords.GetNomNote(i);
	     pl->set_font_description(Pango::FontDescription("sans bold 20"));
	     pl->set_text(sznote);

	     pl->get_pixel_size(wp,hp);
	     retrait=hp/2;

	     gc=Gdk::GC::create(win);
	     xc=width/2+pluspetit/3*cos(M_PI/2-i*M_PI/3);
	     yc=height/2-pluspetit/3*sin(M_PI/2-i*M_PI/3);

	     xline=xc;
	     yline=yc;

	     cr->save();
	     cr->move_to(xline,yline);
	     cr->line_to(xline-pluspetit/taille_trait*cos(M_PI/2-i*M_PI/3),yline+pluspetit/taille_trait*sin(M_PI/2-i*M_PI/3));
	     if (xc<width/2)
		  xc-=wp*1.5;
	     else if (xc>width/2)
		  xc+=wp/2;

	     if ((yc<height/2) && (xc!=width/2))
		  yc-=retrait;

	     if (xc==width/2)
	     {
		  xc-=wp/2;
		  if (yc<height/2)
		       yc-=retrait*3;
		  else
		       yc+=retrait;
	      }

	     if (i==param_affichage.Index_Note_Proche)
	     {
		  gc->set_rgb_fg_color(param_affichage.Couleur);
		  cr->set_source_rgb(param_affichage.Couleur.get_red(),param_affichage.Couleur.get_green(),param_affichage.Couleur.get_blue());

		  if (fabs(param_affichage.Frequence_Pic-Notes_Accords.GetValeurNote(param_affichage.Index_Note_Proche))<0.05)
		  {
		       const int decalage=taille_trait/2;
		       cr->rectangle(xc-decalage,yc-decalage,wp+2*decalage,hp+2*decalage);

		  }
	     }
	     else
	     {
		  gc->set_rgb_fg_color(Noir);
		  cr->set_source_rgb(0,0,0);
	     }

	     win->draw_layout(gc,xc,yc,pl);
	     cr->rectangle(xc,yc,wp,hp);


	     cr->stroke();
	}
//Affichage du niveau
	if (param_affichage.Niveau_Max!=0)
	{

	     position_y_niveau=height/4;

	     for (double i=1E7;((i<param_affichage.Niveau_Max) && (position_y_niveau>0));i*=1.2)
	     {

		  if (i>1E8)
		  {
		       jaune+=0.15;
		       cr->set_source_rgb(1,1-jaune,0);
		  }
		  else
		  {
		       cr->set_source_rgb(rouge,1,0);
		       rouge+=0.15;
		  }

		  cr->move_to(width-width/16,position_y_niveau);
		  cr->line_to(width-width/8,position_y_niveau);

		  position_y_niveau-=5;

		  cr->stroke();
	     }
	}
//Affichage du trait horizontal (précision de la note) et du trait de la note jouée et des couleurs associées
	if (param_affichage.Frequence_Pic!=0)
	{
	     double Angle_Trait,Angle_delta;


	     Angle_delta=((param_affichage.Frequence_Pic-param_affichage.Note_Inferieure)/(param_affichage.Note_Superieure-param_affichage.Note_Inferieure))*M_PI/3;
	     Angle_Trait=M_PI/2-(param_affichage.Index_Note_Inferieure*M_PI/3+Angle_delta);


	     cr->set_source_rgb(param_affichage.Couleur.get_red(),param_affichage.Couleur.get_green(),param_affichage.Couleur.get_blue());
	     cr->move_to(width/2,height/2);
	     cr->line_to(width/2+cos(Angle_Trait)*(pluspetit/3-taille_trait*1.5),height/2-sin(Angle_Trait)*(pluspetit/3-taille_trait*1.5));

	     cr->move_to(width/2,height/2);

	     if (param_affichage.Rapport_precision>1.01)
	     {
		  cr->line_to(width/2+pluspetit/3,height/2);
	     }
	     else if (param_affichage.Rapport_precision<0.99)
		  cr->line_to(width/2-pluspetit/3,height/2);
	     else if ((param_affichage.Rapport_precision>1) && (param_affichage.Rapport_precision<1.01))
		  cr->line_to(width/2+pluspetit/3*(param_affichage.Rapport_precision-1)*100,height/2);
	     else if ((param_affichage.Rapport_precision<1) && (param_affichage.Rapport_precision>0.99))
		  cr->line_to(width/2-pluspetit/3*(1-param_affichage.Rapport_precision)*100,height/2);

	     cr->stroke();
	}

     }

     return true;
}
//Affichage du spectre fréquentiel
bool Gtk_Tuner::on_DrawingSpectre(GdkEventExpose* event)
{
    long int IndexButeemax,IndexButeeMin,hauteur;

    //hauteur du signal
    hauteur=100000;

    Glib::RefPtr<Gdk::Window> win = m_AffichageFFT->get_window();
    if (win)
    {
        Cairo::RefPtr<Cairo::Context> cr =win->create_cairo_context();

        Gtk::Allocation allocation = m_AffichageFFT->get_allocation();

        const int width = allocation.get_width();
        const int height = allocation.get_height();

        if (event)
        {
            //On retrouve la zone a dessiner
            cr->rectangle(event->area.x, event->area.y,
            width, height);
            cr->clip();
        }
        //On met toute la zone en blanc
        cr->save();
        cr->set_source_rgba(1, 1, 1, 0.9);
        cr->paint();
        cr->restore();

        //Si le thread existe et si l'affichage est désiré
        if ((pthread.connected()) && (bAffichageSpectre))
        {

            //Création du type de ligne
            cr->save();
            cr->set_line_cap(Cairo::LINE_CAP_ROUND);
            cr->set_source_rgba(1, 0, 0, 0.9);

            //on retrouve les index min et max des fréquences
            IndexButeemax=(long int)((nMaxi*param_xml.nNbreEchantillon)/param_xml.nFreqEchantillonnage);
            IndexButeeMin=(long int)((nMini*param_xml.nNbreEchantillon)/param_xml.nFreqEchantillonnage);

            //Dessine les lignes représentant les fréquences et leur niveau
            for(long int i=IndexButeeMin;i<IndexButeemax;i++)
            {
                double reel, imaginaire;

                reel=RealO[i];
                imaginaire=ImagO[i];

                cr->move_to(((i-IndexButeeMin)*width)/(IndexButeemax-IndexButeeMin),height);
                cr->line_to((double)((i-IndexButeeMin)*width/(IndexButeemax-IndexButeeMin)),height-sqrt(pow(reel,2)+pow(imaginaire,2))/hauteur);
                cr->stroke();

            }
//Affichage des repères des notes
	    if (param_xml.bVisuRepere)
	    {
		 cr->set_source_rgba(0,0,0,0.9);

		 for (int i=0; i<6; i++)
		 {
		      double Note;
		      int long IndexNote;
		      int Repere;

		      std::string sznote;
		      Glib::RefPtr<Pango::Layout> pl=Pango::Layout::create(cr);

		      Glib::RefPtr<Gdk::GC> gc;

		      sznote=Notes_Accords.GetNomNote(i);
		      pl->set_text(sznote);

		      Note=Notes_Accords.GetValeurNote(i);
		      IndexNote=(long int) ((Note*param_xml.nNbreEchantillon)/param_xml.nFreqEchantillonnage);
		      Repere=((IndexNote-IndexButeeMin)*width)/(IndexButeemax-IndexButeeMin);


		      cr->move_to(Repere,height);
		      cr->line_to((double)((IndexNote-IndexButeeMin)*width/(IndexButeemax-IndexButeeMin)),0);

		      gc=Gdk::GC::create(win);
		      win->draw_layout(gc,Repere-40,30,pl);

		      cr->stroke();

		 }

	    }
             cr->restore();
        }
    }
    return true;
}

//Au changement de la carte son -> Configuration de la carte
int Gtk_Tuner::CarteSon_changed(void)
{
int err;
#ifdef __linux__

    char *nom_carte;
    char *devname=NULL;

    snd_card_get_name(param_xml.nNumero_carte_son,&nom_carte);
    strcat(nom_carte,"\n");

    //chargement des caractéristiques de la carte son
    if ((err=snd_card_load(param_xml.nNumero_carte_son))!=1)
    {
	 std::string message(_("Problem to load the sound card driver "));


        message+=nom_carte;

        message+=snd_strerror(err);
        Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
        Dialog.set_title(_("Sound card problem"));
        Dialog.run();

        err=-1;
    }
    else
    {
        if (handle!=NULL)
            snd_pcm_close(handle);

        devname=new char(100);

        if (devname==NULL)
        {
	     std::string message(_("Memory allocation problem"));

            Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
            Dialog.set_title(_("Memory problem"));
            Dialog.run();

        }
        else
        {

            snprintf(devname,100, ALSA_HARDWARE_CARD, param_xml.nNumero_carte_son);

            if ((err = snd_pcm_open (&handle, devname, SND_PCM_STREAM_CAPTURE, SND_PCM_ASYNC)) < 0)
            {
		 std::string message(_("Problem to open the sound card\n"));


                message+=nom_carte;

                message+=snd_strerror(err);
                Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                Dialog.set_title(_("Sound card problem"));
                Dialog.run();

            }
            else
            {

                if ((err = snd_pcm_hw_params_malloc (&hw_params)) < 0)
                {
		     std::string message(_("Problem to allocate the hardware sound card parameters\n"));

		     message+=nom_carte;

                    message+=snd_strerror(err);
                    Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                    Dialog.set_title(_("Sound card memory problem"));
                    Dialog.run();
                }
                else
                {

                    if ((err = snd_pcm_hw_params_any (handle, hw_params)) < 0)
                    {
			 std::string message(_("Problem to initialise the hardware sound card parameters\n"));


                        message+=nom_carte;

                        message+=snd_strerror(err);
                        Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                        Dialog.set_title(_("Sound card initialisation problem"));
                        Dialog.run();

                    }
                    else
                    {


                        if ((err = snd_pcm_hw_params_set_access (handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
                        {
			     std::string message(_("Problem to update the sound card access mode\n"));


                            message+=nom_carte;

                            message+=snd_strerror(err);
                            Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                            Dialog.set_title(_("Sound card update problem"));
                            Dialog.run();

                        }
                        else
                        {

                            if ((err = snd_pcm_hw_params_set_format (handle, hw_params, SND_PCM_FORMAT_S16_LE)) < 0)
                            {
				 std::string message(_("Problem to set sample sound card format\n"));


                                message+=nom_carte;

                                message+=snd_strerror(err);
                                Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                                Dialog.set_title(_("Sound card format problem"));
                                Dialog.run();

                            }
                            else
                            {

                                snd_pcm_hw_params_get_period_size_max(hw_params,&periodsize,0);

                                //Mise à jour du rate défini dans les paramètres

                                unsigned int rrate=param_xml.nFreqEchantillonnage;

                                if ((err = snd_pcm_hw_params_set_rate_near (handle, hw_params,&rrate, 0)) < 0)
                                {
				     std::string message(_("Problem to set sample sound card rate\n"));


                                    message+=nom_carte;

                                    message+=snd_strerror(err);
                                    Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                                    Dialog.set_title(_("Sound card rate problem"));
                                    Dialog.run();

                                }
                                else
                                {
                                    if ((err=snd_pcm_hw_params_get_channels(hw_params, &periode)<0))
                                    {
					 std::string message(_("Problem to get channel count sound card\n"));

                                        message+=nom_carte;

                                        message+=snd_strerror(err);
                                        Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                                        Dialog.set_title(_("Sound card channel problem"));
                                        Dialog.run();

                                    }
                                    else
                                    {

                                        if ((err = snd_pcm_hw_params_set_channels (handle, hw_params, periode)) < 0)
                                        {
					     std::string message(_("Problem to set channel count sound card\n"));

                                            message+=nom_carte;

                                            message+=snd_strerror(err);
                                            Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                                            Dialog.set_title(_("Sound card channel problem"));
                                            Dialog.run();

                                        }
                                        else
                                        {
                                           
					     if (periodsize>param_xml.nFreqEchantillonnage*periode*2)
						  periodsize=param_xml.nFreqEchantillonnage*periode*2;
                                            //taillebuffer=(periode*periodsize)>>2;
					    taillebuffer=param_xml.nNbreEchantillon*periode*2;
					    snd_pcm_hw_params_set_buffer_size_max(handle,hw_params,&taillebuffer);
					    // snd_pcm_hw_params_set_buffer_size_near(handle,hw_params,&taillebuffer);
                                            snd_pcm_hw_params_set_period_size_near(handle,hw_params,&periodsize,NULL);

                                            if ((err = snd_pcm_hw_params (handle, hw_params)) < 0)
                                            {
						 std::string message(_("Problem to set parameters sound card\n"));


                                                message+=nom_carte;

                                                message+=snd_strerror(err);
                                                Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                                                Dialog.set_title(_("Sound card parameters problem"));
                                                Dialog.run();
                                            }
                                            else
                                            {

                                                snd_pcm_hw_params_free (hw_params);

                                                if ((err = snd_pcm_sw_params_malloc (&sw_params)) < 0)
                                                {
                                                    std::string message("Problem to allocate software parameters structure sound card\n");

                                                    message+=nom_carte;

                                                    message+=snd_strerror(err);
                                                    Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                                                    Dialog.set_title(_("Sound card software parameters problem"));
                                                    Dialog.run();

                                                }
                                                else
                                                {
                                                    if ((err = snd_pcm_sw_params_current (handle, sw_params)) < 0)
                                                    {
							 std::string message(_("Problem to initialise software parameters structure sound card\n"));


                                                        message+=nom_carte;

                                                        message+=snd_strerror(err);
                                                        Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                                                        Dialog.set_title(_("Sound card software parameters problem"));
                                                        Dialog.run();
                                                    }
                                                    else
                                                    {
                                                        if ((err = snd_pcm_sw_params_set_avail_min (handle, sw_params, periode)) < 0)
                                                        {
							     std::string message(_("Problem to set minimum available count sound card\n"));


                                                            message+=nom_carte;

                                                            message+=snd_strerror(err);
                                                            Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                                                            Dialog.set_title(_("Sound card minimum available problem"));
                                                            Dialog.run();
                                                        }
                                                        else
                                                        {

                                                            if ((err = snd_pcm_sw_params_set_start_threshold (handle, sw_params, taillebuffer-periode)) < 0)
                                                            {
								 std::string message(_("Problem to set start mode sound card\n"));

                                                                message+=nom_carte;

                                                                message+=snd_strerror(err);
                                                                Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                                                                Dialog.set_title(_("Sound card start mode problem"));
                                                                Dialog.run();

                                                            }
                                                            else
                                                            {
                                                                if ((err = snd_pcm_sw_params (handle, sw_params)) < 0)
                                                                {
								     std::string message(_("Problem to set software parameters sound card\n"));

                                                                    message+=nom_carte;

                                                                    message+=snd_strerror(err);
                                                                    Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                                                                    Dialog.set_title(_("Sound card software parameters problem"));
                                                                    Dialog.run();


                                                                }

                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            delete devname;
        }

    }

#elif defined(_WIN32)
    //Changement de carte son

    WAVEFORMATEX Format; //Variable sur le format
    MMRESULT resultat; //Resultat de l'opération

    if (Handle!=NULL) //Si on avait déjà un Handle pour la carte son
    {

	 resultat=waveInStop(*Handle); //On arrête l'enregistrement

	 resultat=waveInUnprepareHeader(*Handle,&Buffer,sizeof(Buffer)); //On nettoie le buffer

	 delete Handle; //Suppression du Handle

	 delete Buffer.lpData; //et des données
    }

	 Handle=new HWAVEIN; //Nouvelle allocation pour le Handle

	 Format.wFormatTag=WAVE_FORMAT_PCM; //Mise à jour du format
	 Format.nChannels=1;
	 Format.nSamplesPerSec=param_xml.nFreqEchantillonnage;
	 Format.wBitsPerSample=16;
	 Format.nBlockAlign=(Format.nChannels*Format.wBitsPerSample)/8;
	 Format.nAvgBytesPerSec=Format.nSamplesPerSec*Format.nBlockAlign;
	 Format.cbSize=0;

	 resultat=waveInOpen(Handle,param_xml.nNumero_carte_son,&Format,NULL,NULL,CALLBACK_NULL);
	 //Ouverture pour enregistrement

	 Buffer.dwBufferLength=(unsigned long)(param_xml.nNbreEchantillon*Format.nChannels*Format.wBitsPerSample/8); //Longueur du buffer
	 Buffer.dwFlags=0; //Mise à zéro du flag

	 Buffer.lpData= new char[Buffer.dwBufferLength]; //Allocation des données
	 Buffer.dwBytesRecorded=0; //Pas d'enregistrement pour l'instant

	 taillebuffer=Buffer.dwBufferLength;
	 periode=Format.nBlockAlign;

	 resultat=waveInPrepareHeader(*Handle,&Buffer,sizeof(Buffer)); //Préparation à l'enregistrement

	 if (resultat!=MMSYSERR_NOERROR)
	 {
	      err=-1;
	      std::string message(_("Problem to initialise sound card"));

	      Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
	      Dialog.set_title(message);
	      Dialog.run();
	 }
	 else
	      err=0;

#endif

    return err;

}

void Gtk_Tuner::MAJFrequence(void)
{
    std::ostringstream stream_o;
    std::string szfreq;

    param_affichage.Frequence_Pic=RecherchePic();	//Retourne la fréquence du pic

    stream_o<<param_affichage.Frequence_Pic;
    szfreq=_("Frequency:")+stream_o.str();
    stsInfo[4].push(szfreq,ctx[4]);

    param_affichage.Index_Note_Inferieure=Notes_Accords.GetNoteInf(param_affichage.Frequence_Pic);	//Retourne l'index de la note de l'accord immédiatement inférieur
    param_affichage.Index_Note_Superieure=Notes_Accords.GetNoteSup(param_affichage.Frequence_Pic);	//Retourne l'index de la note de l'accord immédiatement supérieur

    if (param_affichage.Index_Note_Inferieure>-1)	//Si l'index inférieur fait partie de l'accord
    {
	 param_affichage.Note_Inferieure=Notes_Accords.GetValeurNote(param_affichage.Index_Note_Inferieure);	//Retourne la valeur de la note

    }
    else	//Sinon la note est inférieure
    {
	 param_affichage.Note_Inferieure=0;	//Note inférieure est égale à zéro
    }

    if (param_affichage.Index_Note_Superieure<6)	//Si l'index supérieur fait partie de l'accord
    {
	 param_affichage.Note_Superieure=Notes_Accords.GetValeurNote(param_affichage.Index_Note_Superieure);	//Retourne la valeur de la note

    }
    else	//Sinon
    {
	 //Note supérieure est déduite de la valeur de la position maximale du Slider et de la note maximale de l'accord
	 param_affichage.Note_Superieure=Notes_Accords.GetValeurNote(5)*2-Notes_Accords.GetValeurNote(4);

    }

    if (abs(int(param_affichage.Note_Superieure-param_affichage.Frequence_Pic))<abs(int(param_affichage.Note_Inferieure-param_affichage.Frequence_Pic)))
    {
	 //Si supérieur, le slider est à fond à droite
	 param_affichage.Index_Note_Proche=param_affichage.Index_Note_Superieure;
    }

    else
    {
	 //Sinon à fond à gauche
	 param_affichage.Index_Note_Proche=param_affichage.Index_Note_Inferieure;
    }

//Mise à jour des couleurs des labels en fonction de la position du Slider
     if ((param_affichage.Frequence_Pic/Notes_Accords.GetValeurNote(param_affichage.Index_Note_Proche)>=0.995) && (param_affichage.Frequence_Pic/Notes_Accords.GetValeurNote(param_affichage.Index_Note_Proche)<=1.005))
     {
	  //Vert lorsque la note est > 0.995 et < 1.005
	  param_affichage.Couleur.set_rgb(0,65535,0);
	  param_affichage.bcouleur=true;

     }
     else if ((param_affichage.Frequence_Pic/Notes_Accords.GetValeurNote(param_affichage.Index_Note_Proche)>=0.99) && (param_affichage.Frequence_Pic/Notes_Accords.GetValeurNote(param_affichage.Index_Note_Proche)<=1.01))
     {
	  //Jaune entre 0.99 et 1.01
	 param_affichage.Couleur.set_rgb(65535,65535,0);
	 param_affichage.bcouleur=true;

     }
     else
     {
	  //Aucune Couleur sinon
	  param_affichage.Couleur.set_rgb(0,0,0);
	  param_affichage.bcouleur=false;

     }
     param_affichage.Rapport_precision=param_affichage.Frequence_Pic/Notes_Accords.GetValeurNote(param_affichage.Index_Note_Proche);
}


//Maj de la position du slider
void Gtk_Tuner::MAJSlider(void)
{
     //Mise à jour de la position de l'index du Slider
     int MarqueInf,MarqueSup;
     double Facteur;
     int PositionSlider;

     if (param_affichage.Index_Note_Inferieure>-1)	//Si l'index inférieur fait partie de l'accord
     {
	  MarqueInf=nPositionPremierIndex+param_affichage.Index_Note_Inferieure*nDistanceInterIndex;	//Retourne la position de l'index correspondant
     }
     else
	  MarqueInf=0;

     if (param_affichage.Index_Note_Superieure<6)	//Si l'index supérieur fait partie de l'accord
     {
	  MarqueSup=nPositionPremierIndex+param_affichage.Index_Note_Superieure*nDistanceInterIndex;	//Retourne la position de l'index correspondant
     }
     else
     {
	 //Index Supérieur égale à la position max du Slider
	  MarqueSup=hsNiveau.get_adjustment()->get_upper();
     }

     Facteur=(param_affichage.Frequence_Pic-param_affichage.Note_Inferieure)/(param_affichage.Note_Superieure-param_affichage.Note_Inferieure);
     PositionSlider=((MarqueSup-MarqueInf)*Facteur+MarqueInf)*(hsNiveau.get_adjustment()->get_upper()/(nPositionPremierIndex*2+nDistanceInterIndex*5));

     //Mise à jour de la position du slider précision
     MAJSliderPrecision(param_affichage.Rapport_precision);

     hsNiveau.get_adjustment()->set_value(PositionSlider);	//Positionne le Slider

     AffichageCouleur(param_affichage.Index_Note_Proche,param_affichage.Couleur,param_affichage.bcouleur);

}

//Recherche du pic de la fréquence
double Gtk_Tuner::RecherchePic(void)
{
     double picTemp=0,buteemax,pic;
    long int i=1,IndexPic=0,IndexButeemax;
    bool bNoteTrouve=FALSE;

    double niveau;

    param_affichage.Niveau_Max=0;

    //On retrouve la note butée du Slider
    buteemax=Notes_Accords.GetValeurNote(5)*2-Notes_Accords.GetValeurNote(4);

	//L'index dans le tableau d'échantillons
    IndexButeemax=(long int)((buteemax*param_xml.nNbreEchantillon)/param_xml.nFreqEchantillonnage);

    //On retrouve la valeur de l'atténuation
    niveau=param_xml.nValeurAttenuation*400000;

    while (/*(!bNoteTrouve) && */(i<IndexButeemax))
    {

        picTemp=sqrt(pow(RealO[i],2)+pow(ImagO[i],2));
       //Si le pic temporaire est supérieur à un niveau

        if (picTemp>param_affichage.Niveau_Max)
        {

	   param_affichage.Niveau_Max=picTemp;
           if (picTemp>niveau)
           {


               if ((bNoteTrouve=ValidationPic(i))==true)	//si le pic est valide
                {
                    //On met à jour l'index dans le tableau
                    pic=picTemp;

                    IndexPic=i;
                }
           }
        }
     i++;
    }

    double freq;

    //Retourne la fréquence du pic
    freq=(double)(IndexPic*param_xml.nFreqEchantillonnage)/param_xml.nNbreEchantillon;

    return freq;
}

//Recherche si le pic est valide
bool Gtk_Tuner::ValidationPic(int IndexFrequenceCentrale)
{
    bool retour=false;
	int j=1;

	if (IndexFrequenceCentrale>=20)	//si l'index est sup à 20
	{
		// et que l'amplitude index-1 et +1 est inférieur à l'ampl de l'index

		retour=true;

        //recherche aux alentours du pic -20 +20 qu'il n'existe aucun supérieur
		while ((retour==true) && (j<20))
		{
			if ((sqrt(pow(RealO[IndexFrequenceCentrale-j],2)+pow(ImagO[IndexFrequenceCentrale-j],2))<sqrt(pow(RealO[IndexFrequenceCentrale],2)+pow(ImagO[IndexFrequenceCentrale],2))) && (sqrt(pow(RealO[IndexFrequenceCentrale+j],2)+pow(ImagO[IndexFrequenceCentrale+j],2))<sqrt(pow(RealO[IndexFrequenceCentrale],2)+pow(ImagO[IndexFrequenceCentrale],2))))
				retour=true;	//alors vrai
			else retour=false;
			j++;
		}
	}


	return retour;

}

//Affichage des couleurs des labels
void Gtk_Tuner::AffichageCouleur(int IndexLabel,Gdk::Color Couleur, bool bCouleur)
{

        //Pour tous les labels pas de couleurs
        for (int i=0;i<6;i++)
        {


           Eventbox[i].modify_style(style);


       }

        if (bCouleur)
        {
	    Eventbox[IndexLabel].modify_bg(Gtk::STATE_NORMAL,Couleur);
        }

}


//Mise à jour du Slider précidion
void Gtk_Tuner::MAJSliderPrecision(double Delta)
{
     if (Delta>1.01)
     {
	  hsPrecis.set_value(100);
     }
     else if (Delta<0.99)
	  hsPrecis.set_value(0);
     else if ((Delta>1) && (Delta<1.01))
	  hsPrecis.set_value(hsPrecis.get_adjustment()->get_upper()/2+(Delta-1)*5000);
     else if ((Delta<1) && (Delta>0.99))
	  hsPrecis.set_value(hsPrecis.get_adjustment()->get_upper()/2-(1-Delta)*5000);

        //Couleur vert si le note est parfaite (jamais réussi)
        if (Delta==1)
        {
            Gdk::Color Couleur;

            Couleur.set_rgb(0,65535,0);
	    Eventbox[6].modify_bg(Gtk::STATE_NORMAL,Couleur);
        }
        else
	    Eventbox[6].modify_style(style);

}
//Appui sur le menu Quitter
void Gtk_Tuner::on_Quit()
{
    on_wndGuitarTuner_hide();
}
//Appui sur le menu propriétés
void Gtk_Tuner::on_menuProprietes()
{

    onproprietes();
}
//Chargement des paramètres par le fichier xml
void Gtk_Tuner::Chargement_parametres(std::string szFichierParam)
{
    bool ret;   //retour sur les erreurs
    const char *valeur; //valeur du paramètre
    xmlDocPtr xmlparametres_doc = NULL; //pointeur sur le document
    xmlXPathContextPtr xmlparametres_context = NULL;    //Pointeur sur le contexte
    xmlXPathObjectPtr xmlobject;    //Pointeur sur l'objet
    xmlNodePtr node;    //noeud


    ret=false;
    xmlparametres_doc = xmlParseFile (szFichierParam.c_str());  //On retrouve le pointeur sur le document

    if (xmlparametres_doc)
    {

        xmlXPathInit ();    //Initialisation des chemins

        xmlparametres_context = xmlXPathNewContext (xmlparametres_doc); //On retrouve le pointeur sur le contexte

        if (xmlparametres_context)
        {
            ret=true;
            //On retrouve le premier objet
            xmlobject = xmlXPathEval (BAD_CAST "/PARAMETRES/HEADER/Visualisation_Spectrale/text()", xmlparametres_context);

            if (xmlobject->nodesetval)
            {
                //le noeud
                node = xmlobject->nodesetval->nodeTab[0];
                //et la valeur associée
                valeur=(const char*)node->content;
                //Mise à jour du paramètre associé
                param_xml.bVisuFreq=atoi(valeur);
            }
            //meme chose pour les autres paarmètres
            xmlobject = xmlXPathEval (BAD_CAST "/PARAMETRES/HEADER/Frequence_Echantillonnage/text()", xmlparametres_context);

            if (xmlobject->nodesetval)
            {
                node = xmlobject->nodesetval->nodeTab[0];

                valeur=(const char*)node->content;

                param_xml.nFreqEchantillonnage=atoi(valeur);
            }

            xmlobject = xmlXPathEval (BAD_CAST "/PARAMETRES/HEADER/Nombre_Echantillon/text()", xmlparametres_context);

            if (xmlobject->nodesetval)
            {
                node = xmlobject->nodesetval->nodeTab[0];

                valeur=(const char*)node->content;

                param_xml.nNbreEchantillon=atoi(valeur);
            }

            xmlobject = xmlXPathEval (BAD_CAST "/PARAMETRES/HEADER/Numero_Accord/text()", xmlparametres_context);

            if (xmlobject->nodesetval)
            {
                node = xmlobject->nodesetval->nodeTab[0];

                valeur=(const char*)node->content;

                param_xml.nNumeroAccord=atoi(valeur);
            }

            xmlobject = xmlXPathEval (BAD_CAST "/PARAMETRES/HEADER/Numero_Carte_Son/text()", xmlparametres_context);

            if (xmlobject->nodesetval)
            {
                node = xmlobject->nodesetval->nodeTab[0];

                valeur=(const char*)node->content;

                param_xml.nNumero_carte_son=atoi(valeur);
            }

            xmlobject = xmlXPathEval (BAD_CAST "/PARAMETRES/HEADER/Valeur_Attenuation/text()", xmlparametres_context);

            if (xmlobject->nodesetval)
            {
                node = xmlobject->nodesetval->nodeTab[0];

                valeur=(const char*)node->content;

                param_xml.nValeurAttenuation=atoi(valeur);
            }
	    xmlobject = xmlXPathEval (BAD_CAST "/PARAMETRES/HEADER/Interface_Graphique/text()", xmlparametres_context);
	    if (xmlobject->nodesetval)
            {
                node = xmlobject->nodesetval->nodeTab[0];

                valeur=(const char*)node->content;

                param_xml.nInterface_Affichage=atoi(valeur);
            }
	    xmlobject = xmlXPathEval (BAD_CAST "/PARAMETRES/HEADER/Frequence_Repere/text()", xmlparametres_context);
	    if (xmlobject->nodesetval)
            {
                node = xmlobject->nodesetval->nodeTab[0];

                valeur=(const char*)node->content;

                param_xml.bVisuRepere=atoi(valeur);
            }

            xmlXPathFreeObject(xmlobject);
            xmlXPathFreeContext(xmlparametres_context);
        }
        else
        {
	     std::string message(_("Problem to create a xml context"));
            Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
            Dialog.set_title(_("XML Problem"));
            Dialog.run();
        }

         xmlFreeDoc(xmlparametres_doc); //libération de la mémoire

    }

    if (!ret)   //Si problème lors de la lecture des paramètres
    {
        if (param_xml.bFichierOK)   //Si le fichier existe
        {
            //On garde les anciens parametres
	     std::string message(_("Cannot read the parameters\n"));
	     message+=_("Old parameters are kept");
	     Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
	     Dialog.set_title(_("Unknown parameters"));
	     Dialog.run();
        }
        else
        {
            //Si pas d'ancien parametres -> pas d'analyse
	    std::string message(_("Cannot read the parameters\n"));
	    message+=_("The analyse is impossible");
            Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
            Dialog.set_title(_("Unknown parameters"));
            Dialog.run();
            //Controle analyse inactif
	    m_refActionGroup->get_action("Analyse")->set_sensitive(false);
        }
    }
}
//Mise à jour du chemin du chemin du fichier accord.xml
void Gtk_Tuner::SetPathAccordXML(std::string szChemin)
{
    m_szchemin_accord_xml=szChemin;
}
//Lors du chargement des propriétés
void Gtk_Tuner::Chargement_Proprietes(void)
{
    xmlTextReaderPtr reader;    //Pointeur sur le fichier paramètres

    reader=xmlReaderForFile(m_szparametres_xml.c_str(),NULL,0); //On retrouve le pointeur

    m_refGlade->get_widget_derived("dialog1",m_dlgProprietes);  //Mise à jour du pointeur pour la boite de dialogue

    if (m_dlgProprietes)
    {
        int retour;

        retour=m_dlgProprietes->Chargement_Liste_Carte_Son();   //Chargement des cartes sons

        if (retour==-1)
            bCarteSon=false;
        else
            bCarteSon=true;


        retour=m_dlgProprietes->Chargement_Accords(m_szchemin_accord_xml);  //Chargement des accords

        if (retour==-1)
            bAccord=false;
        else
            bAccord=true;
    }

    if (reader!=NULL)
    {

        MAJControles(); //Si fichier paramètres existe, maj des controles

    }
    else
    {

        param_xml.bFichierOK=false;
#ifdef __linux__
	struct stat st;
        if(stat(m_szHome.c_str(),&st) != 0) //Si le dossier .guitar_tuner n'existe pas
        {
            if (mkdir(m_szHome.c_str(),0777) !=0)   //Création du dossier
            {
		 std::string message(_("Cannot create the .guitar_tuner directory\n"));
		 message+=_("The software stopped");

                Gtk::MessageDialog::MessageDialog Dialog(message.c_str(),false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CLOSE,false );
                Dialog.set_title(_("Problem to create directory"));
                Dialog.run();

                on_wndGuitarTuner_hide();
            }
        }
#endif
        onproprietes(); //Ouverture de la boite de dialogue propriétés

    }

    xmlFreeTextReader(reader);  //Libération de la mémoire

}
//Maj des controles
void Gtk_Tuner::MAJControles(void)
{
     char *carac;
     std::ostringstream stream_o;

     if ((param_xml.nInterface_Affichage==0) && (Vbox1.get_children().size()!=2))
	  Vbox1.remove(Vbox3);
     else if ((param_xml.nInterface_Affichage==1) && (Vbox1.get_children().size()!=2))
	  Vbox1.remove(m_Affichage_Rond);
    //Chargement des paramètres avec le fichier
    Chargement_parametres(m_szparametres_xml);

    if (CarteSon_changed()<0)
    {
        //Si problème lors de l'ouverture de la carte son pas d'analyse
	 m_refActionGroup->get_action("Analyse")->set_sensitive(false);
    }
    else
    {
        param_xml.bFichierOK=true;

        if (Accord_changed()<0)
        {
            //pareil pour le fichier des accords
	   m_refActionGroup->get_action("Analyse")->set_sensitive(false);
        }

    }

    //on retrouve le nom de la carte son
#ifdef __linux__
    snd_card_get_name(param_xml.nNumero_carte_son,&carac);
#elif defined (_WIN32)
WAVEINCAPS caracteristiques;	//Variable pour les caractéristiques de la carte son

waveInGetDevCaps(param_xml.nNumero_carte_son,&caracteristiques,sizeof(WAVEINCAPS));	//On récupère les caractéristiques de la carte son
carac=caracteristiques.szPname;

#endif

   std::string temp(carac),carte;

   

   carte=_("Sound card: ") + temp;
   stsInfo[0].push(carte,ctx[0]); //mise a jour du statusbar
   temp=_("Chord: ")+ Notes_Accords.GetNomAccord();
   
   stsInfo[1].push(temp,ctx[1]); //mise a jour dcu statusbar

   stream_o<<param_xml.nFreqEchantillonnage;    //frequence d'echantillonnage
   temp=_("Sample: ") + stream_o.str();
   stsInfo[2].push(temp,ctx[2]); //mise a jour du statusbar

   stream_o.str("");

   stream_o<<param_xml.nNbreEchantillon;    //Nombre d'echantillons
   temp=_("Samples: ") + stream_o.str();
   stsInfo[3].push(temp,ctx[3]); //mise à jour du statusbar
//Selon le type d'affichage, mise à jour du concerné
   switch(param_xml.nInterface_Affichage)
   {
   case 0:
	Affichage_Standard();
	break;
   case 1:
	Affichage_Rond();
	break;
   default:
	Affichage_Standard();

   }

}
//Affichage de la boite de dialogue A propos de
void Gtk_Tuner::on_menuAPropos()
{
    m_refGlade->get_widget("aboutdialog1",m_dlgAPropos);  //Mise à jour du pointeur pour la boite de dialogue

    m_dlgAPropos->run();

    m_dlgAPropos->hide();
}
