#include <gtkmm.h>
#include "Proprietes.h"
#include <libxml/xmlreader.h>
#include <libxml/xpath.h>
#include "Accord.h"
#include "Variables.h"
#ifdef __linux__
#include <asoundlib.h>
#endif //#ifdef __linux__
#include <glibmm/thread.h>



#ifndef GTK_TUNER_H_INCLUDED
#define GTK_TUNER_H_INCLUDED


//Classe de la fenetre principale
class Gtk_Tuner : public Gtk::Window
{
public:
    Gtk_Tuner(const Glib::RefPtr<Gnome::Glade::Xml> & refGlade);   //Constructeur
    ~Gtk_Tuner();   //Destucteur
    void MAJSlider(void);   //Maj du slider des notes
    int AnalyseSignal(void);    //Thread de l'analyse du signal
    void Chargement_Proprietes(void);   //Chargement des parametres
    void SetPathAccordXML(std::string szChemin); //Mise à jour de la variable du chemin du fichier accord xml
   
//Tableaux pour la transformée de Fourier
    double *ImagO;
    double *RealO;
    double *ImagIn;
    double *RealIn;

#ifdef __linux__
    //variable pour la taille du buffer et la période
    snd_pcm_uframes_t taillebuffer;
    snd_pcm_hw_params_t *hw_params; //Handle pour le hardware
    snd_pcm_sw_params_t *sw_params; // Handle pour le software
    snd_pcm_uframes_t periodsize;   //Taille du buffer nécessare
#elif defined (_WIN32)
    unsigned long taillebuffer;
#endif

    unsigned int periode;

    Gtk::Allocation alloc;  //Pour la position de l'image de l'index

protected:

    virtual void onbuttonarret();   //Arret de l'analyse
    virtual void onbuttonanalyser();    //Analyse le signal
    virtual void onproprietes();    //Affichage de la boite de dialogue propriétés
    virtual void on_wndGuitarTuner_hide();  //Lors de la fin du programme
    virtual bool on_DrawingSpectre(GdkEventExpose* event);  //Mise à jour de l'affichage de la fenetre spectrale
    virtual bool on_DrawingRond(GdkEventExpose* event);

    virtual void  on_Quit();    //Lors de l'appui sur Quitter
    virtual void  on_menuProprietes(); //Lors de l'appui sur le menu propriétés
    virtual void on_menuAPropos(); //Lors de l'appui a propos de

private :
    CAccord Notes_Accords;  //Variables contenant les accords
    std::vector<int> arsndCartes;   //Vecteur des index des cartes son
    Glib::RefPtr<Gnome::Glade::Xml> m_refGlade; //Pointeur sur le fichier Glade
//Les différentes parties de l'affichage standard
    Gtk::VBox Vbox1,Vbox2,Vbox3;
    Gtk::HBox Hbox1,Hbox2,Hbox3,Hbox4,Hbox5,Hbox6;
//Paramètres d'affichage
    parametres_affichage param_affichage;
//Pour les menus et raccourcis
    Glib::RefPtr<Gtk::UIManager> m_refUIManager;
    Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
//Les différents contrôles
    Gtk::EventBox Eventbox[7];
    Gtk::Label lblNotes[6];
    Gtk::Image Image[7];
    Gtk::HScale hsNiveau,hsPrecis;
    Gtk::Adjustment adjustement,Adjustment_precis;
    Gtk::Statusbar stsInfo[5];
    unsigned ctx[5];

    Gtk::DrawingArea *m_AffichageFFT;   //La fenetre d'affichage
    Gtk::DrawingArea m_Affichage_Rond; //La fenêtre avec l'interface ronde

    Gtk_Proprietes *m_dlgProprietes;    //Pointeur sur la boite de dialogue proprietes
    Gtk::AboutDialog *m_dlgAPropos;   //Pointeur sur la boite de dialogue a propos de

    Gtk::Window *m_wndSpectre;  //Pointeur sur la fenetre spectrale

    int nPositionPremierIndex, nDistanceInterIndex; //Variables de position des index
    Glib::RefPtr<Gtk::RcStyle> style;   //style par défaut des labels

    sigc::connection pthread;  //Handle du thread de l'analyse

    double RecherchePic(void);  //Recherche du Pic
    bool ValidationPic(int IndexFrequenceCentrale); //Validation du pic
    void AffichageCouleur(int IndexLabel,Gdk::Color Couleur,bool bCouleur);   //Affichage de la couleur pour le label des notes
    void MAJSliderPrecision(double Delta);  //Maj du slider de précision
    void Chargement_parametres(std::string szFichierParam);   //Fonction pour le chargement des paramètres du fichier de sauvegarde
    int CarteSon_changed(void);  //Lors du changement de la carte son
    int Accord_changed(void);    //Lors du changement d'accord
    void MAJControles(void);    //Maj de la carte son et des labels
    void Affichage_Standard(void);//Affichage des contrôles standard
    void Affichage_Rond(void); //Affichage des accords sur un rond
    void MAJFrequence(void);//Mise à jour de la fréquence de la note
    void Demarrage_thread(void); //Démarrage du thread

    unsigned long int IndexEchantillon;   //Nb d'échantillons lues depuis le début
    unsigned long int nPowerEchantillons;   //Nb d'échantillons total à  enregistrer

    int nMaxi;  //Note maxi pour la fenetre de visualisation du spectre
    int nMini;  //Note mini
    bool bAffichageSpectre; //Affichage du spectre

    std::string m_szchemin_accord_xml;  //Chemin du fichier accord xml;
    std::string m_szparametres_xml; //Chemin du fichier guitar_tuner.xml
    std::string m_szchemin_share; //Chemin des images et icones
    std::string m_szHome;

    parametres param_xml;   //Structure sur les parametres

    bool bCarteSon; //booléen sur la validité d'une carte son
    bool bAccord;   //booléen sur la validité d'un accord

};
#endif // GTK_TUNER_H_INCLUDED
